"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const EventBusValidator_1 = require("./EventBusValidator");
const fs_1 = (0, tslib_1.__importDefault)(require("fs"));
const graphql_1 = require("graphql");
const path_1 = (0, tslib_1.__importDefault)(require("path"));
const graphql_tag_1 = (0, tslib_1.__importDefault)(require("graphql-tag"));
const events_1 = require("events");
const mock_1 = require("@graphql-tools/mock");
const graphql_scalars_1 = require("graphql-scalars");
const validator_1 = require("./validator");
const typeDef = fs_1.default.readFileSync(path_1.default.join(__dirname, "../data/events.graphql"), "utf-8");
const publisherSchema = (0, graphql_1.buildSchema)(typeDef);
const mockSchema = (0, mock_1.addMocksToSchema)({
    schema: publisherSchema,
    mocks: graphql_scalars_1.mocks,
});
const validator = new validator_1.Validator(mockSchema);
describe("EventBusValidator", () => {
    test("consumer works as expected for accepted and rejected consumer message", () => (0, tslib_1.__awaiter)(void 0, void 0, void 0, function* () {
        const publishCb = jest.fn();
        const consumeCb = jest.fn().mockResolvedValue(null);
        const subscribeCb = jest.fn();
        const getTopicCb = jest.fn();
        const signupEventEmitter = new events_1.EventEmitter();
        const entityFlagEmitter = new events_1.EventEmitter();
        const queries = (0, graphql_tag_1.default) `
      query SignUpEvent {
        SignUpEvent {
          id
        }
      }
      query EntityFlagEvent {
        EntityFlagEvent {
          id
          groupId
        }
      }
    `;
        const consumerEventBus = new EventBusValidator_1.EventBusValidator({
            publisherSchema,
        });
        yield consumerEventBus.validateConsumerQueries(queries);
        expect(yield consumerEventBus.extractData({
            topic: "SignUpEvent",
            data: validator.sample("SignUpEvent").data["SignUpEvent"],
        })).toMatchObject({
            id: expect.any(String),
        });
    }));
    test("publisher works as expected", () => (0, tslib_1.__awaiter)(void 0, void 0, void 0, function* () {
        const eventBusValidator = new EventBusValidator_1.EventBusValidator({
            publisherSchema,
        });
        yield expect(eventBusValidator.publishValidate({
            topic: "SignUpEvent",
            payload: validator.sample("SignUpEvent").data["SignUpEvent"],
        })).resolves;
        // undefined event name
        yield expect(eventBusValidator.publishValidate({
            topic: "UnknownEvent",
            payload: validator.sample("SignUpEvent").data["SignUpEvent"],
        })).rejects;
        // wrong payload
        yield expect(eventBusValidator.publishValidate({
            topic: "SignUpEvent",
            payload: { id: 5 },
        })).rejects;
    }));
});
