"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MemoryEventBus = void 0;
const tslib_1 = require("tslib");
const events_1 = require("events");
const VanillaEventBus_1 = require("./VanillaEventBus");
class MemoryEventBus {
    constructor(config) {
        this.config = config;
        this.eventEmitter = new events_1.EventEmitter();
        this.init = () => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            yield this.bus.init();
        });
        this.publish = (args) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            yield this.bus.publish({
                topic: args.topic,
                payload: args.payload,
            });
        });
        this.close = () => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            this.eventEmitter.removeAllListeners();
        });
        this.eventEmitter.setMaxListeners(100000);
        this.bus = new VanillaEventBus_1.VanillaEventBus({
            plugins: config.plugins,
            publisher: {
                schema: config.schema,
                publish: (args, ...rest) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
                    this.eventEmitter.emit(`message-${args.topic}`, JSON.stringify(args.baggage), ...rest);
                }),
                allowInvalidTopic: config.allowInvalidTopic,
            },
            subscriber: this.config.subscriber
                ? {
                    cb: this.config.subscriber.cb,
                    getSubscription: (topic, cb) => {
                        this.eventEmitter.on(`message-${topic}`, (baggage) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
                            yield cb(JSON.parse(baggage));
                        }));
                    },
                    queries: this.config.subscriber.queries,
                    schema: this.config.schema,
                }
                : undefined,
        });
    }
}
exports.MemoryEventBus = MemoryEventBus;
