"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const LoggingPlugin_1 = require("./LoggingPlugin");
const waait_1 = (0, tslib_1.__importDefault)(require("waait"));
const graphql_tag_1 = (0, tslib_1.__importDefault)(require("graphql-tag"));
const fs_1 = (0, tslib_1.__importDefault)(require("fs"));
const path_1 = (0, tslib_1.__importDefault)(require("path"));
const graphql_1 = require("graphql");
const MemoryEventBus_1 = require("./MemoryEventBus");
const mock_1 = require("@graphql-tools/mock");
const graphql_scalars_1 = require("graphql-scalars");
const _1 = require(".");
const uuid_1 = require("uuid");
const typeDef = fs_1.default.readFileSync(path_1.default.join(__dirname, "../data/events.graphql"), "utf-8");
const publisherSchema = (0, graphql_1.buildSchema)(typeDef);
const mockSchema = (0, mock_1.addMocksToSchema)({
    schema: publisherSchema,
    mocks: graphql_scalars_1.mocks,
});
const validator = new _1.Validator(mockSchema);
describe("MemoryEventBus", () => {
    test("works", () => (0, tslib_1.__awaiter)(void 0, void 0, void 0, function* () {
        const consumeCb = jest.fn();
        const queries = (0, graphql_tag_1.default) `
      query Query1 {
        SignUpEvent {
          id
        }
      }
      query Query2 {
        EntityFlagEvent {
          id
          groupId
        }
      }
    `;
        const subscribeConfig = {
            cb: consumeCb,
            queries,
            schema: publisherSchema,
        };
        const bus = new MemoryEventBus_1.MemoryEventBus({
            schema: publisherSchema,
            subscriber: subscribeConfig,
            plugins: [LoggingPlugin_1.LoggingPlugin],
        });
        yield bus.init();
        yield bus.publish({
            topic: "SignUpEvent",
            payload: validator.sample("SignUpEvent").data["SignUpEvent"],
            metadata: { test: "data" },
        });
        yield (0, waait_1.default)(0);
        expect(consumeCb).toBeCalledTimes(1);
        expect(consumeCb.mock.calls[0][0]).toMatchObject({
            payload: {
                id: expect.any(String),
            },
            topic: "SignUpEvent",
            metadata: { test: "data" },
        });
        // not existing topic cannot be published
        expect(bus.publish({
            topic: "A",
            payload: {
                id: (0, uuid_1.v4)(),
                timestamp: new Date().toISOString(),
            },
        })).rejects.toThrow();
        yield bus.publish({
            topic: "EntityFlagEvent",
            payload: validator.sample("EntityFlagEvent").data["EntityFlagEvent"],
            metadata: { "x-request-id": "123" },
        });
        yield (0, waait_1.default)(0);
        expect(consumeCb.mock.calls[1][0]).toMatchObject({
            payload: {
                id: expect.any(String),
                groupId: expect.any(String),
            },
            topic: "EntityFlagEvent",
            metadata: { "x-request-id": "123" },
        });
        consumeCb.mockClear();
        // not subscribed events are not consumed
        yield bus.publish({
            topic: "Complex",
            payload: validator.sample("Complex").data["Complex"],
        });
        yield (0, waait_1.default)(0);
        expect(consumeCb).not.toBeCalled();
    }));
});
